﻿using ERP.Framework.Exceptions;
using ERP.Framework.WebApi;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.Filters;
using Newtonsoft.Json;
using Serilog;
using System.Net;

namespace ERP.Framework.Filter
{
    public class GlobalExceptionFilter : IAsyncExceptionFilter
    {
        /// <summary>
        /// 异步异常处理
        /// </summary>
        /// <param name="context"></param>
        /// <returns></returns>
        public Task OnExceptionAsync(ExceptionContext context)
        {
            if (context.ExceptionHandled)
            {
                return Task.CompletedTask;
            }

            var result = new Result();

            if (context.Exception is HttpException)
            {
                var ex = (HttpException)context.Exception;
                result.Code = ex.Code;
                result.Message = ex.Message;
                context.Result = CreateContentResult(result, ex.HttpCode);
            }
            else if (context.Exception is CodeException)
            {
                var ex = (CodeException)context.Exception;
                result.Code = ex.Code;
                result.Message = ex.Message;
                context.Result = CreateContentResult(result);
            }
            else
            {
                var ex = context.Exception;
                result.Code = ErrorCode.ERROR;
                result.Message = ex.Message;
                context.Result = CreateContentResult(result, HttpStatusCode.InternalServerError);
            }

            Log.Error("Message:{0} Source:{1} StackTrace:{2}", context.Exception.Message, context.Exception.Source, context.Exception.StackTrace);

            context.ExceptionHandled = true;
            return Task.CompletedTask;
        }

        /// <summary>
        /// 创建返回对象
        /// </summary>
        /// <param name="httpCode">Http 状态码</param>
        /// <param name="response">返回Body</param>
        /// <returns></returns>
        private static ContentResult CreateContentResult(Result response, HttpStatusCode httpCode = HttpStatusCode.OK)
        {
            return new ContentResult
            {
                StatusCode = (int)httpCode,
                ContentType = "application/json;charset=utf-8",
                Content = JsonConvert.SerializeObject(response)
            };
        }
    }
}